{% if previously_reported or can_report %}
    <div class="report-comment-abuse">
        <span>
            {% if previously_reported %}
                Abuse Reported
            {% else %}
                <a class="reporter-comment-abuse" href="{% url like content_type content_obj.id vote %}">Report Abuse</a>
            {% endif %}
        </span>
    </div>
{% endif %}
